unit BeeGui_Forms;

{ Contains:

  TBGCustomForm class
  TBGExtractForm class
  TBGAddForm class
  TBGConfigForm  class
  TBGMainForm class

  (C) 2005 Andrew Filinsky, Melchiorre Caruso

  Modifyed:

  v1.0.1 build 9155 - 2005/07/15 Melchiorre Caruso
  v1.0.1 build 9158 - 2005/07/24 Melchiorre Caruso

}

{$R-,Q-,S-}

interface

uses
   Forms,
   Menus,
   Classes,
   Dialogs,
   Buttons,
   IniFiles,
   STDCtrls,
   SysUtils,
   ExtCTRLS,
   COMCtrls,
   Controls,
   Graphics;

const
  languageext = '.language.ini';
  settingext  = '.setting.ini';

type
   TBGCustomForm = class (TForm)
   public
     constructor Create (AOwner: TComponent); override;
     destructor  Destroy; override;
   end;

   TBGExtractForm = class (TBGCustomForm)
   public
     destructor Destroy; override;
   end;

   TBGAddForm = class (TBGCustomForm)
   public
     destructor Destroy; override;
   end;

   TBGIntViewerForm = class (TBGCustomForm)
   public
     destructor Destroy; override;
   end;

   TBGConfigForm = class (TBGCustomForm)
   public
     destructor SaveAndDestroy; 
   end;

   TBGMainForm = class (TForm)
   public
     constructor Create (AOwner: TComponent); override;
     procedure Save (HideForm: boolean); 
   end;

  // Functions and procedures
  function   GetIntegerValue (const Section,Ident: ansistring; usereg: boolean): integer;
  function   GetStrValue     (const Section,Ident: ansistring; usereg: boolean): ansistring;
  function   GetBoolValue    (const Section,Ident: ansistring; usereg: boolean): boolean;


implementation

uses
  BeeGui_WinOS,
  BeeGui_ListView;

var
  REG: TMemIniFile;
  LNG: TIniFile;

    // Registry procedures

    procedure   OpenRegistry;
    begin
      try
        REG := TMemIniFile.Create (ChangeFileExt (ParamStr (0), settingext));
        LNG := TIniFile.Create (ChangeFileExt (ParamStr (0), languageext));
      except
        Halt;
      end;
    end;

    procedure  CloseRegistry;
    begin
      REG.Free;
      LNG.Free;
    end;

    procedure  SaveRegistry;
    var
      S: TStringList;
    begin
      S := TStringList.Create;
      try
        REG.GetStrings (S);
        S.SaveToFile (REG.FileName);
      finally
        S.Free;
      end;
    end;

    // Opcode extention

    procedure  SaveTBGListView (List: TBGListView);
    var
      I: Integer;
    begin
      for I := 0 to List.Columns.Count - 1 do
        REG.WriteInteger (List.ClassName, List.Name + '.Column[' + IntToStr(I) + '].Width', List.Columns [I].Width);
      REG.WriteInteger (List.ClassName, List.Name + '.ColumnSort', List.ColumnSort);
      REG.WriteBool    (List.ClassName, List.Name + '.ListMode', List.ListMode);
    end;

    procedure  LoadTBGListView (List: TBGListView);
    var
      I: integer;
    begin
      for I := 0 to List.Columns.Count - 1 do
      begin
        if LNG.ValueExists (List.ClassName, List.Name + '.Column[' + IntToStr(I) + '].Caption') then
          List.Columns [I].Caption := LNG.ReadString (List.ClassName, List.Name + '.Column[' + IntToStr(I) + '].Caption', List.Columns [I].Caption)
        else
          LNG.WriteString (List.ClassName, List.Name + '.Column[' + IntToStr(I) + '].Caption', List.Columns [I].Caption);
        List.Columns [I].Width := REG.ReadInteger (List.ClassName, List.Name + '.Column[' + IntToStr(I) + '].Width', List.Columns [I].Width);
      end;
      List.ColumnSort := REG.ReadInteger (List.ClassName, List.Name + '.ColumnSort', List.ColumnSort);
      List.ListMode   := REG.ReadBool    (List.ClassName, List.Name + '.ListMode', List.ListMode);
    end;

    procedure SaveTMenuItem (Item: TMenuItem);
    begin
      REG.WriteBool (TMenuItem.ClassName, Item.Name + '.Checked', Item.Checked);
    end;

    procedure LoadTMenuItem (Item: TMenuItem);
    begin
      if LNG.ValueExists (TMenuItem.ClassName, Item.Name + '.Caption') then
        Item.Caption := LNG.ReadString (TMenuItem.ClassName, Item.Name + '.Caption', Item.Caption)
      else
        LNG.WriteString (TMenuItem.ClassName, Item.Name + '.Caption', Item.Caption);
      Item.Checked := REG.ReadBool (TMenuItem.ClassName, Item.Name + '.Checked', Item.Checked);
    end;

    procedure  LoadTMainMenu (Menu: TMainMenu);
    var
      I: integer;
    begin
      for I := 0 to Menu.Items.Count - 1 do
        LoadTMenuItem (Menu.Items [I]);
    end;

    procedure  SaveTMainMenu (Menu: TMainMenu);
    var
      I: integer;
    begin
      for I := 0 to Menu.Items.Count - 1 do
        SaveTMenuItem (Menu.Items [I]);
    end;

    procedure  SaveTEdit (Edit: TEdit);
    begin
      REG.WriteString (Edit.ClassName, Edit.Name + '.Text', Edit.Text);
    end;

    procedure  LoadTEdit (Edit: TEdit);
    begin
      Edit.Text := REG.ReadString (Edit.ClassName, Edit.Name + '.Text', Edit.Text);
    end;

    procedure  SaveTCheckBox (CB: TCheckBox);
    begin
      REG.WriteBool (CB.ClassName, CB.Name + '.Checked', CB.Checked);
    end;

    procedure  LoadTCheckBox (CB: TCheckBox);
    begin
      if LNG.ValueExists (CB.ClassName, CB.Name + '.Caption') then
        CB.Caption := LNG.ReadString (CB.ClassName, CB.Name + '.Caption', CB.Caption)
      else
        LNG.WriteString (CB.ClassName, CB.Name + '.Caption', CB.Caption);
      CB.Checked := REG.ReadBool (CB.ClassName, CB.Name + '.Checked', CB.Checked);
    end;

    procedure  SaveTRadioButton (RB: TRadioButton);
    begin
      REG.WriteBool (RB.ClassName, RB.Name + '.Checked', RB.Checked);
    end;

    procedure  LoadTRadioButton (RB: TRadioButton);
    begin
      if LNG.ValueExists (RB.ClassName, RB.Name + '.Caption') then
        RB.Caption := LNG.ReadString (RB.ClassName, RB.Name + '.Caption', RB.Caption)
      else
        LNG.WriteString (RB.ClassName, RB.Name + '.Caption',  RB.Caption);
      RB.Checked := REG.ReadBool (RB.ClassName, RB.Name + '.Checked', RB.Checked);
    end;

    procedure  SaveTBGComboBox (CB: TComboBox);
    var
      I: integer;
    begin
      for I := 0 to CB.Items.Count -1 do
        REG.WriteString (CB.ClassName, CB.Name + '.Items[' + IntToStr(I) + ']', CB.Items [I]);

      REG.WriteInteger (CB.ClassName, CB.Name + '.ItemIndex',  CB.ItemIndex);
    end;

    procedure  LoadTBGComboBox (CB: TComboBox);
    var
      I: integer;
    begin
      for I := 0 to CB.Items.Count - 1 do
        if REG.ValueExists (CB.ClassName, CB.Name + '.Items[' + IntToStr(I) + ']') then
          CB.Items [I] := REG.ReadString (CB.ClassName, CB.Name + '.Items[' + IntToStr(I) + ']', CB.Items [I])
        else
          REG.WriteString (CB.ClassName, CB.Name + '.Items[' + IntToStr(I) + ']', CB.Items [I]);
      CB.ItemIndex := REG.ReadInteger (CB.ClassName, CB.Name + '.ItemIndex', 0);
    end;

    procedure  SaveTComboBox (CB: TComboBox);
    begin
      REG.WriteInteger (CB.ClassName, CB.Name + '.ItemIndex',  CB.ItemIndex);
    end;

    procedure  LoadTComboBox (CB: TComboBox);
    var
      I: integer;
    begin
      for I := 0 to CB.Items.Count - 1 do
        if LNG.ValueExists (CB.ClassName, CB.Name + '.Items[' + IntToStr(I) + ']') then
          CB.Items [I] := LNG.ReadString (CB.ClassName, CB.Name + '.Items[' + IntToStr(I) + ']', CB.Items [I])
        else
          LNG.WriteString (CB.ClassName, CB.Name + '.Items[' + IntToStr(I) + ']', CB.Items [I]);
      CB.ItemIndex := REG.ReadInteger (CB.ClassName, CB.Name + '.ItemIndex', 2);
    end;

    procedure  SaveTSaveDialog (Dialog: TSaveDialog);
    begin
      REG.WriteString (TCommonDialog.ClassName, Dialog.Name + '.InitialDir', Dialog.InitialDir);
    end;

    procedure  LoadTSaveDialog (Dialog: TSaveDialog);
    begin
      if LNG.ValueExists (TCommonDialog.ClassName, Dialog.Name + '.Title') then
        Dialog.Title := LNG.ReadString (TCommonDialog.ClassName, Dialog.Name + '.Title', Dialog.Title)
      else
        LNG.WriteString (TCommonDialog.ClassName, Dialog.Name + '.Title', Dialog.Title);

      Dialog.InitialDir := REG.ReadString (TCommonDialog.ClassName, Dialog.Name + '.InitialDir', Dialog.InitialDir);
    end;

    procedure  SaveTOpenDialog (Dialog: TOpenDialog);
    begin
      REG.WriteString (TCommonDialog.ClassName, Dialog.Name + '.InitialDir', Dialog.InitialDir);
    end;

    procedure  LoadTOpenDialog (Dialog: TOpenDialog);
    begin
      if LNG.ValueExists (TCommonDialog.ClassName, Dialog.Name + '.Title') then
        Dialog.Title := LNG.ReadString (TCommonDialog.ClassName, Dialog.Name + '.Title', Dialog.Title)
      else
        LNG.WriteString (TCommonDialog.ClassName, Dialog.Name + '.Title', Dialog.Title);

      Dialog.InitialDir := REG.ReadString (TCommonDialog.ClassName, Dialog.Name + '.InitialDir', Dialog.InitialDir);
    end;

    procedure  SaveTFontDialog (Dialog: TFontDialog);
    begin
      REG.WriteInteger (Dialog.ClassName, Dialog.Name + '.Charset', Integer (Dialog.Font.Charset));
      REG.WriteInteger (Dialog.ClassName, Dialog.Name + '.Color'  , Integer (Dialog.Font.Color  ));
      REG.WriteString  (Dialog.ClassName, Dialog.Name + '.Name'   ,          Dialog.Font.Name    );
      REG.WriteInteger (Dialog.ClassName, Dialog.Name + '.Height' ,          Dialog.Font.Height  );
      REG.WriteInteger (Dialog.ClassName, Dialog.Name + '.Pitch'  , Integer (Dialog.Font.Pitch  ));
      REG.WriteInteger (Dialog.ClassName, Dialog.Name + '.Size'   ,          Dialog.Font.Size    );

      if (fsBold in Dialog.Font.Style) then
        REG.WriteBool (Dialog.ClassName, Dialog.Name + '.Style[fsBold]', True)
      else
        REG.WriteBool (Dialog.ClassName, Dialog.Name + '.Style[fsBold]', False);

      if (fsItalic in Dialog.Font.Style) then
        REG.WriteBool (Dialog.ClassName, Dialog.Name + '.Style[fsItalic]', True)
      else
        REG.WriteBool (Dialog.ClassName, Dialog.Name + '.Style[fsItalic]', False);

      if (fsUnderline in Dialog.Font.Style) then
        REG.WriteBool (Dialog.ClassName, Dialog.Name + '.Style[fsUnderline]', True )
      else
        REG.WriteBool (Dialog.ClassName, Dialog.Name + '.Style[fsUnderline]', False);

      if (fsStrikeOut in Dialog.Font.Style) then
        REG.WriteBool (Dialog.ClassName, Dialog.Name + '.Style[fsStrikeOut]', True)
      else
        REG.WriteBool (Dialog.ClassName, Dialog.Name + '.Style[fsStrikeOut]', False);
    end;

    procedure  LoadTFontDialog (Dialog: TFontDialog);
    begin
      Dialog.Font.Charset := TFontCharSet (REG.ReadInteger (Dialog.ClassName, Dialog.Name + '.Charset', Dialog.Font.Charset));
      Dialog.Font.Color := TColor (REG.ReadInteger (Dialog.ClassName, Dialog.Name + '.Color', Dialog.Font.Color));
      Dialog.Font.Name := REG.ReadString (Dialog.ClassName, Dialog.Name + '.Name', Dialog.Font.Name);
      Dialog.Font.Height := REG.ReadInteger (Dialog.ClassName, Dialog.Name + '.Height', Dialog.Font.Height);
      Dialog.Font.Pitch := TFontPitch (REG.ReadInteger (Dialog.ClassName, Dialog.Name  +'.Pitch', Integer (Dialog.Font.Pitch)));
      Dialog.Font.Size := REG.ReadInteger (Dialog.ClassName, Dialog.Name + '.Size', Dialog.Font.Size);

      if REG.ReadBool (Dialog.ClassName, Dialog.Name + '.Style[fsBold]', False) then
        Dialog.Font.Style := Dialog.Font.Style + [fsBold]
      else
        Dialog.Font.Style := Dialog.Font.Style - [fsBold];

      if REG.ReadBool (Dialog.ClassName, Dialog.Name + '.Style[fsItalic]', False) then
        Dialog.Font.Style := Dialog.Font.Style + [fsItalic]
      else
        Dialog.Font.Style := Dialog.Font.Style - [fsItalic];

      if REG.ReadBool (Dialog.ClassName, Dialog.Name + '.Style[fsUnderline]', False) then
        Dialog.Font.Style := Dialog.Font.Style + [fsUnderline]
      else
        Dialog.Font.Style := Dialog.Font.Style - [fsUnderline];

      if REG.ReadBool (Dialog.ClassName, Dialog.Name + '.Style[fsStrikeOut]', False) then
        Dialog.Font.Style := Dialog.Font.Style + [fsStrikeOut]
      else
        Dialog.Font.Style := Dialog.Font.Style - [fsStrikeOut];
    end;

    procedure  LoadTGroupBox (GB: TGroupBox);
    begin
      if LNG.ValueExists (GB.ClassName, GB.Name + '.Caption') then
       GB.Caption := LNG.ReadString (GB.ClassName, GB.Name + '.Caption', GB.Caption)
      else
       LNG.WriteString (GB.ClassName, GB.Name + '.Caption', GB.Caption);
    end;

    procedure  LoadTLabel (L: TLabel);
    begin
      if LNG.ValueExists (TLabel.ClassName, L.Name + '.Caption') then
        L.Caption := LNG.ReadString (TLabel.ClassName, L.Name + '.Caption', L.Caption)
      else
        LNG.WriteString (TLabel.ClassName, L.Name + '.Caption', L.Caption);
    end;

    procedure  LoadTTreeView (Tree: TTreeView);
    var
      I: integer;
    begin
      for I := 0 to Tree.Items.Count - 1 do
        if LNG.ValueExists (Tree.ClassName, Tree.Name + '.Items[' + IntToStr (I) + '].Text') then
          Tree.Items [I].Text := LNG.ReadString (Tree.ClassName, Tree.Name + '.Items[' + IntToStr (I) + '].Text', Tree.Items [I].Text)
        else
          LNG.WriteString (Tree.ClassName, Tree.Name + '.Items[' + IntToStr (I) + '].Text', Tree.Items [I].Text);
    end;

    procedure  LoadTPageControl (PageC: TPageControl);
    var
      I: Integer;
    begin
      for I := 0 to PageC.PageCount - 1 do
        if LNG.ValueExists (TPageControl.ClassName, PageC.Pages [i].Name + '.Caption') then
          PageC.Pages [I].Caption := LNG.ReadString (TPageControl.ClassName, PageC.Pages [i].Name + '.Caption', PageC.Pages [i].Caption)
        else
          LNG.WriteString (TPageControl.ClassName, PageC.Pages [i].Name + '.Caption', PageC.Pages [i].Caption);
    end;

    procedure  SaveTListView (List:TLISTVIEW);
    var
      I: Integer;
    begin
      for I := 0 to List.Columns.Count - 1 do
        REG.WriteInteger (List.ClassName, List.Name + '.Column[' + IntToStr(I) + '].Width', List.Columns [I].Width);
    end;

    procedure  LoadTListView (List: TLISTVIEW);
    var
      I: integer;
    begin
      for I := 0 to List.Columns.Count - 1 do
      begin
        if LNG.ValueExists (List.ClassName, List.Name + '.Column[' + IntToStr(I) + '].Caption') then
          List.Columns [I].Caption := LNG.ReadString (List.ClassName, List.Name + '.Column[' + IntToStr(I) + '].Caption', List.Columns [I].Caption)
        else
          LNG.WriteString (List.ClassName, List.Name + '.Column[' + IntToStr(I) + '].Caption', List.Columns [I].Caption);
        List.Columns [I].Width := REG.ReadInteger (List.ClassName, List.Name + '.Column[' + IntToStr(I) + '].Width', List.Columns [I].Width);
      end;
    end;

    procedure  LoadTBitBtn (Btn: TBitBtn);
    begin
      if LNG.ValueExists (TBitBtn.ClassName, Btn.Name + '.Caption') then
        Btn.Caption := LNG.ReadString (TBitBtn.ClassName, Btn.Name + '.Caption', Btn.Caption)
      else
        LNG.WriteString(TBitBtn.ClassName, Btn.Name + '.Caption', Btn.Caption);
    end;

    procedure  LoadTSpeedButton (Btn: TSpeedButton);
    begin
      if LNG.ValueExists (TSpeedButton.ClassName, Btn.Name + '.Caption') then
        Btn.Caption := LNG.ReadString (TSpeedButton.ClassName, Btn.Name + '.Caption', Btn.Caption)
      else
        LNG.WriteString(TSpeedButton.ClassName, Btn.Name + '.Caption', Btn.Caption);
    end;

    function   GetStrValue (const Section,Ident: ansistring; usereg: boolean): ansistring;
    begin
      if usereg then
        Result := REG.ReadString (Section, Ident, '')
      else
        Result := LNG.ReadString (Section, Ident, '');
    end;

    function  GetBoolValue (const Section,Ident: ansistring; usereg: boolean): boolean;
    begin
      if usereg then
        Result := REG.ReadBool (Section, Ident, False)
      else
        Result := LNG.ReadBool (Section, Ident, False);
    end;

    function  GetIntegerValue (const Section,Ident: ansistring; usereg: boolean): integer;
    begin
      if usereg then
        Result := REG.ReadInteger (Section, Ident, -1)
      else
        Result := LNG.ReadInteger (Section, Ident, -1);
    end;

    procedure SaveTForm (Form: TForm);
    begin
      if (Form.WindowState = wsNormal) then
      begin
        if (Form.BorderStyle = bsSizeable) then
        begin
          REG.WriteInteger (TForm.ClassName, Form.Name + '.Heigth', Form.Height);
          REG.WriteInteger (TForm.ClassName, Form.Name + '.Width', Form.Width);
        end;
        REG.WriteInteger (TForm.ClassName, Form.Name + '.Left', Form.Left);
        REG.WriteInteger (TForm.ClassName, Form.Name + '.Top', Form.Top);
      end;
      REG.WriteInteger (TForm.ClassName, Form.Name + '.WState', Integer (Form.WindowState));
    end;

    procedure  LoadTForm (Form: TForm);
    begin
      if LNG.ValueExists (TForm.ClassName, Form.Name + '.Caption') then
        Form.Caption := LNG.ReadString (TForm.ClassName, Form.Name + '.Caption', Form.Caption)
      else
        LNG.WriteString (TForm.ClassName, Form.Name + '.Caption', Form.Caption);

      if (Form.BorderStyle = bsSizeable) then
      begin
        Form.Height := REG.ReadInteger (TForm.ClassName, Form.Name + '.Heigth', Form.Height);
        Form.Width := REG.ReadInteger (TForm.ClassName, Form.Name + '.Width', Form.Width);
        Form.WindowState := TWindowState (REG.ReadInteger (TForm.ClassName, Form.Name + '.WState', 0));
      end;
      Form.Left := REG.ReadInteger (TForm.ClassName, Form.Name + '.Left', Form.Left);
      Form.Top := REG.ReadInteger (TForm.ClassName, Form.Name + '.Top', Form.Top);
    end;

    procedure  SaveTControl (Control:TControl);
    begin
      if Control.ClassType = TMenuItem    then SaveTMenuItem    (TMenuItem    (Control)) else
      if Control.ClassType = TSaveDialog  then SaveTSaveDialog  (TSaveDialog  (Control)) else
      if Control.ClassType = TOpenDialog  then SaveTOpenDialog  (TOpenDialog  (Control)) else
      if Control.ClassType = TFontDialog  then SaveTFontDialog  (TFontDialog  (Control)) else
      if Control.ClassType = TCheckBox    then SaveTCheckBox    (TCheckBox    (Control)) else
      if Control.ClassType = TBGComboBox  then SaveTBGComboBox  (TBGComboBox  (Control)) else
      if Control.ClassType = TComboBox    then SaveTComboBox    (TComboBox    (Control)) else
      if Control.ClassType = TEdit        then SaveTEdit        (TEdit        (Control)) else
      if Control.ClassType = TRadioButton then SaveTRadioButton (TRadioButton (Control)) else
      if Control.ClassType = TListView    then SaveTListView    (TListView    (Control)) else
      if Control.ClassType = TBGListView  then SaveTBGListView  (TBGListView  (Control));
    end;

    procedure  LoadTControl (Control:TControl);
    begin
      if Control.ClassType = TSpeedButton then LoadTSpeedButton (TSpeedButton (Control)) else
      if Control.ClassType = TBitBtn      then LoadTBitBtn      (TBitBtn      (Control)) else
      if Control.ClassType = TMenuItem    then LoadTMenuItem    (TMenuItem    (Control)) else
      if Control.ClassType = TLabel       then LoadTLabel       (TLabel       (Control)) else
      if Control.ClassType = TSaveDialog  then LoadTSaveDialog  (TSaveDialog  (Control)) else
      if Control.ClassType = TOpenDialog  then LoadTOpenDialog  (TOpenDialog  (Control)) else
      if Control.ClassType = TFontDialog  then LoadTFontDialog  (TFontDialog  (Control)) else
      if Control.ClassType = TGroupBox    then LoadTGroupBox    (TGroupBox    (Control)) else
      if Control.ClassType = TCheckBox    then LoadTCheckBox    (TCheckBox    (Control)) else
      if Control.ClassType = TRadioButton then LoadTRadioButton (TRadioButton (Control)) else
      if Control.ClassType = TBGComboBox  then LoadTBGComboBox  (TBGComboBox  (Control)) else
      if Control.ClassType = TComboBox    then LoadTComboBox    (TComboBox    (Control)) else
      if Control.ClassType = TEdit        then LoadTEdit        (TEdit        (Control)) else
      if Control.ClassType = TTreeView    then LoadTTreeView    (TTreeView    (Control)) else
      if Control.ClassType = TPageControl then LoadTPageControl (TPageControl (Control)) else
      if Control.ClassType = TListView    then LoadTListView    (TListView    (Control)) else
      if Control.ClassType = TBGListView  then LoadTBGListView  (TBGListView  (Control)) ;
    end;

// TBeeGuiCustomForm

constructor TBGCustomForm.Create;
var
  I: integer;
begin
  inherited Create (AOwner);
  LoadTForm (TForm (Self));
  for I := 0 to Self.ComponentCount - 1 do
    LoadTControl (TControl (Self.Components [I]));
end;

destructor TBGCustomForm.Destroy;
begin
  SaveTForm (TForm (Self));
  inherited Destroy;
end;

// TBeeGuiExtractForm

destructor TBGExtractForm.Destroy;
var
  I: integer;
begin
  for I := 0 to Self.ComponentCount - 1 do
  begin
    if AnsiCompareText(TControl (Self.Components [I]).Name, 'ExtractFrm_ExtrDir') = 0 then
      SaveTControl (TControl (Self.Components [I]));
  end;
  inherited Destroy;
end;

// TBeeGuiAddForm

destructor TBGAddForm.Destroy;
var
  I: integer;
begin
  for I := 0 to Self.ComponentCount - 1 do
  begin
    if AnsiCompareText(TControl (Self.Components [I]).Name, 'AddFrm_FilesList') = 0 then
      SaveTControl (TControl (Self.Components [I]));
  end;
  inherited Destroy;
end;

// TBeeGuiIntViewerForm

destructor TBGIntViewerForm.Destroy;
var
  I: integer;
begin
  for I := 0 to Self.ComponentCount - 1 do
    if TControl (Self.Components [I]).ClassType = TFontDialog then
      SaveTControl (TControl (Self.Components [I]));
  inherited Destroy;
end;

// TBeeGuiConfigForm

destructor TBGConfigForm.SaveAndDestroy;
var
  I: integer;
begin
  for I := 0 to ComponentCount - 1 do
     SaveTControl (TControl (Components [I]));
  inherited Destroy;
end;

// TBeeGuiMainForm

constructor TBGMainForm.Create;
var
  I: integer;
begin
  inherited Create (AOwner);
  LoadTForm (TForm (Self));
  for I := 0 to Self.ComponentCount - 1 do
    LoadTControl (TControl (Self.Components [I]));
end;

procedure TBGMainForm.Save;
var
  I: integer;
begin
  Visible := not HideForm;
  SaveTForm (TForm (Self));
  for I := 0 to ComponentCount - 1 do
    SaveTControl (TControl (Components [I]));

  SaveRegistry;
end;


// initialization and finalization    

initialization OpenRegistry;

finalization CloseRegistry;


end.

