unit BeeGui_WinOS;

{ Contains:

  TBGFileInfo class
  Various helper sustem routines

  (C) 2005 Andrew Filinsky, Melchiorre Caruso

  Modifyed:

  v1.0.1 build 9147 - 2005/07/11 Melchiorre Caruso
  v1.0.1 build 9154 - 2005/07/24 Melchiorre Caruso
  v1.0.1 build 9160 - 2005/08/03 Melchiorre Caruso

}

{$R-,Q-,S-}

interface

uses
  ShlObj,
  ActiveX,
  Dialogs,
  Windows,
  Classes,
  ShellApi,
  Controls,
  SysUtils,
  Registry;
  

type
  TBGFileInfo = record
    IconIndex: Integer;
    SelectIconIndex: Integer;
    TypeName: string;
  end;

procedure BG_LoadListViewImages (Small, Large: TImageList);

function  BG_DiskInDrive (Drive: Char): boolean;
procedure BG_GetDrivers (var DriveList: TStringList);
function  BG_GetFileImageIndex (const Str: string; const A: integer): integer;
function  BG_GetFileOpenImageIndex (const Str: string; const A: integer): integer;
function  BG_GetDirList (const Dir: string; var DirList: TStringList): boolean;
procedure BG_GetFileInfo (const Str: string; A: integer; var FI: TBGFileInfo);
function  BG_ShellExecute (const FileName: string; const PathName: string): integer;
procedure BG_ShellExecuteAndWait (const FileName: string; const PathName: string; Wait: boolean);

function  BG_GetCurrentUserName: string;
procedure BG_RegisterFileType (prefix: string; exepfad: string);
procedure BG_UnRegisterFileType (prefix: string; exepfad: string);
function  BG_ShellFindExecutable (const FileName, DefaultDir: string): string;

function  BG_GetWin_UserTmpDir: string;
function  BG_GetWin_StartMenuDir: string;
function  BG_GetWin_StartMenuProgramsDir: string;
function  BG_GetWin_DesktopDir: string;

// file version routines

function  BG_FileVersion (const FileName: string): string;
function  BG_CompareVersion (const VerA, VerB: string): boolean;
procedure BG_ExpandVersion (Version : string; out Major, Minor, Release, Build: Integer);

implementation

uses
  Bee_Common;

function BG_DiskInDrive;
  var ErrorMode: word;
begin
  if Drive in ['a'..'z'] then Dec (Drive, $20);
  if (Drive in ['A'..'Z']) then
  begin
    ErrorMode := SetErrorMode (SEM_FailCriticalErrors);
    try
      if DiskSize (Ord (Drive) - $40) = -1 then
        Result:=False
      else
        Result:=True;
    finally
      SetErrorMode (ErrorMode);
    end;
  end
  else
    Result := False;
end;

function BG_GetDirList;
  var Info: TSearchRec;
       err: Integer;
begin
  Result := False;
  DirList.Clear;

  if not BG_DiskInDrive (Dir [1]) then Exit;
  if not SetCurrentDir (Dir) then Exit;

  err := SysUtils.FindFirst ('*.*' ,faDirectory, Info);
  while err = 0 do
  begin
    if ((Info.Attr and faHidden) = 0) and
       ((Info.Attr and faDirectory) = faDirectory) and
       (Info.Name <> '..') and
       (Info.Name <> '.') then
      DirList.Add(Info.Name);

    err := SysUtils.FindNext (Info);
  end;
  SysUtils.FindClose (Info);

  Result := True;
end;

procedure BG_GetDrivers;
  var i, PX: Integer;
    TPC: PChar;
    TMR, DM: string;
begin
  DM := '';
  DriveList.Clear;
  GetMem (TPC, 100);
  PX := GetLogicalDriveStrings (100, TPC);
  for i := - 1 to PX do if (TPC + i) [1] <> #0 then DM := DM + (TPC + i) [1];
  FreeMem (TPC);
  TMR := '';
  for i := 1 to Length (DM) do
  begin
    TMR := TMR + DM [i];
    if (i mod 3) = 0 then begin
      DriveList.Add (TMR);
      TMR := '';
    end;
  end;
end;

function BG_ShellExecute;
begin
  Result := ShellExecute (0, 'open', PChar (FileName), nil, PChar (PathName), SW_SHOW);
end;

procedure BG_ShellExecuteAndWait;
var
  SEI: TShellExecuteInfo;
begin
  FillChar (SEI, SizeOf (SEI), #0);
  SEI.cbSize := SizeOf (SEI);
  if Wait then
    SEI.fMask := SEE_MASK_NOCLOSEPROCESS;
  SEI.lpFile := PChar (FileName);
  SEI.lpDirectory := PChar (PathName);
  SEI.nShow := SW_NORMAL;

  if ShellExecuteEx (@SEI) then begin
    if Wait then
      WaitForSingleObject (SEI.hProcess, INFINITE);
    CloseHandle (SEI.hProcess);
  end;
end;

procedure BG_GetFileInfo;
  var SFI: TSHFileInfo;
begin
  SHGetFileInfo (PChar (Str), A, SFI, SizeOf (TSHFileInfo), SHGFI_SYSICONINDEX or SHGFI_TYPENAME or SHGFI_USEFILEATTRIBUTES);

  FI.IconIndex := SFI.iIcon;
  FI.TypeName := SFI.szTypeName;
end;

function BG_GetFileImageIndex;
  var SFI: TSHFileInfo;
begin
  SHGetFileInfo (PChar (Str), A, SFI, SizeOf (TSHFileInfo), SHGFI_SYSICONINDEX or SHGFI_TYPENAME or SHGFI_USEFILEATTRIBUTES);

  Result := SFI.iIcon;
end;

function BG_GetFileOpenImageIndex;
  var SFI: TSHFileInfo;
begin
  SHGetFileInfo (PChar (Str), A, SFI, SizeOf (TSHFileInfo), SHGFI_SYSICONINDEX or SHGFI_TYPENAME or SHGFI_USEFILEATTRIBUTES or SHGFI_OPENICON);

  Result := SFI.iIcon;
end;

procedure BG_LoadListViewImages;
var
  SysImageList: uint;
  SFI: TSHFileInfo;
begin
  SysImageList := SHGetFileInfo ('', 0, SFI, SizeOf (TSHFileInfo), SHGFI_SYSICONINDEX or SHGFI_SMALLICON);
  if SysImageList <> 0 then
  begin
    Small.Handle := SysImageList;
    Small.ShareImages := TRUE;
  end;

  SysImageList := SHGetFileInfo ('', 0, SFI, SizeOf (TSHFileInfo), SHGFI_SYSICONINDEX or SHGFI_LARGEICON);
  if SysImageList <> 0 then
  begin
    Large.Handle := SysImageList;
    Large.ShareImages := TRUE;
  end;
end;

function BG_GetCurrentUserName;
const
  cnMaxUserNameLen = 254;
var
  sUserName: string;
  dwUserNameLen: DWORD;
begin
  dwUserNameLen := cnMaxUserNameLen - 1;
  SetLength (sUserName, cnMaxUserNameLen);
  GetUserName (PChar (sUserName), dwUserNameLen);
  SetLength (sUserName, dwUserNameLen);
  Result := sUserName;
end;

procedure BG_RegisterFileType;
var
  reg: TRegistry;
begin
  reg := TRegistry.Create;
  try
    reg.RootKey := HKEY_CLASSES_ROOT;                                  

    reg.OpenKey('.' + prefix, True);                                   //create a new key  --> .pci
    try
      reg.Writestring('', prefix + 'file');                            //create a new value for this key --> pcifile
    finally
      reg.CloseKey;
    end;

    reg.CreateKey(prefix + 'file');                                    //create a new key --> pcifile
    reg.OpenKey(prefix + 'file\DefaultIcon', True);                    //create a new key pcifile\DefaultIcon
    try
      reg.Writestring('', exepfad + ',0');                             //and create a value where the icon is stored --> c:\project1.exe,0
    finally
      reg.CloseKey;
    end;

    reg.OpenKey(prefix + 'file\shell\open\command', True);             // Shell Open archive
    try
      reg.Writestring('', exepfad+' "o" "%1"');
    finally
      reg.CloseKey;
    end;

    reg.OpenKey(prefix + 'file\shell\Extract all...\command', True);   // Shell Extract archive
    try
      reg.Writestring('', exepfad+' "e" "%1"');
    finally
      reg.CloseKey;
    end;
    
  finally
    reg.Free;
  end;

  SHChangeNotify (SHCNE_ASSOCCHANGED, SHCNF_IDLIST, nil, nil);
end;

procedure BG_UnRegisterFileType;
var
  reg: TRegistry;
begin
  reg := TRegistry.Create;
  try
    reg.RootKey := HKEY_CLASSES_ROOT;
    if (reg.openKey (prefix + 'file\shell\open\command', False) = True) and (PosText (exepfad,reg.readstring ('')) = 1) then
    begin
      reg.CloseKey;
      reg.DeleteKey ('.' + prefix);
      reg.DeleteKey (prefix + 'file');
    end;
  finally 
    reg.Free;
  end;
  SHChangeNotify(SHCNE_ASSOCCHANGED, SHCNF_IDLIST, nil, nil);
end;

procedure FreePidl(pidl: PItemIDList);
var
  allocator: IMalloc;
begin 
  if Succeeded (SHGetMalloc (allocator)) then
  begin
    allocator.Free (pidl);
    {$IFDEF VER100}
    allocator.Release;
    {$ENDIF}
  end;
end;

function BG_GetWin_StartMenuDir;
var
  pidl: PItemIDList;
  buf: array [0..MAX_PATH] of Char;
begin
  if Succeeded (SHGetSpecialFolderLocation (0, CSIDL_STARTMENU , pidl)) then SHGetPathFromIDList (pidl, buf);
  Result := StrPas (buf);
  // The calling application is responsible for freeing the PItemIDList-pointer
  // with the Shell's IMalloc interface
  FreePIDL (pidl);
end;

function BG_GetWin_StartMenuProgramsDir;
var
  pidl: PItemIDList;
  buf: array[0..MAX_PATH] of char;
begin
  if Succeeded (SHGetSpecialFolderLocation (0, CSIDL_PROGRAMS, pidl)) then SHGetPathFromIDList(pidl, buf);
  Result := StrPas(buf);
  // The calling application is responsible for freeing the PItemIDList-pointer
  // with the Shell's IMalloc interface
  FreePIDL (pidl);
end;

function BG_GetWin_DesktopDir;
var
  pidl: PItemIDList;
  buf: array[0..MAX_PATH] of char;
begin
  if Succeeded (SHGetSpecialFolderLocation (0, CSIDL_DESKTOP, pidl)) then SHGetPathFromIDList(pidl, buf);
  Result := StrPas (buf);
  // The calling application is responsible for freeing the PItemIDList-pointer
  // with the Shell's IMalloc interface
  FreePIDL (pidl);
end;

function BG_GetWin_UserTmpDir;
var
  tmppath: pchar;
begin
  getmem (tmppath, max_path);
  GetTempPath (max_path, tmppath);
  result := IncludeDelimiter (string (tmppath));
  freemem (tmppath, max_path);
end;

function BG_ShellFindExecutable;
var
  Res: HINST;
  Buffer: array[0..MAX_PATH] of char;
  P: PChar;
begin
  FillChar (Buffer, SizeOf (Buffer), #0);
  if DefaultDir = '' then P := nil
  else
    P := PChar (DefaultDir);
  Res := FindExecutable (PChar (FileName), P, Buffer);
  if Res > 32 then
  begin
    P := Buffer;
    while PWord (P)^ <> 0 do
    begin
      if P^ = #0 then // FindExecutable replaces #32 with #0
        P^ := #32;
      Inc (P);
    end;
    Result := Buffer;
  end
  else
    Result := '';
end;

// file version routines

function BG_FileVersion;
var
  InfoSize, Wnd: DWORD;
  VerBuf: pointer;
  FI: PVSFixedFileInfo;
  VerSize: DWORD;
begin
  Result := '0.0.0.0';
  InfoSize := GetFileVersionInfoSize (PChar(FileName), Wnd);
  if InfoSize <> 0 then
  begin
    GetMem (VerBuf, InfoSize);
    try
      if GetFileVersionInfo (PChar (FileName), Wnd, InfoSize, VerBuf) then
        if VerQueryValue (VerBuf, '\', Pointer (FI), VerSize) then

          Result := IntToStr(FI.dwFileVersionMS div $10000) + '.' +
                    IntToStr(FI.dwFileVersionMS mod $10000) + '.' +
                    IntToStr(FI.dwFileVersionLS div $10000) + '.' +
                    IntToStr(FI.dwFileVersionLS mod $10000);
    finally
      FreeMem (VerBuf);
    end;
  end;
end;

function BG_CompareVersion;
var
  AMajor, AMinor, ARelease, ABuild: integer;
  BMajor, BMinor, BRelease, BBuild: integer;
begin
  BG_ExpandVersion (VerA, AMajor, AMinor, ARelease, ABuild);
  BG_ExpandVersion (VerB, BMajor, BMinor, BRelease, BBuild);
  if AMajor = BMajor then
  begin
    if AMinor = BMinor then
    begin
      if ARelease = BRelease then
        Result := ABuild < BBuild
      else
        Result := ARelease < BRelease
    end
    else
      Result := AMinor < BMinor;
  end
  else
    Result := AMajor < BMajor;
end;

procedure BG_ExpandVersion;

  function ExtractNumber (var Version: string): integer;
  var
    p: integer;
  begin
    p := Pos ('.', Version);
    if p > 0 then
    begin
      Result :=
        StrToIntDef (Copy (Version, 1, p - 1), 0);
      Delete (Version, 1, p);
    end
    else
      Result := StrToIntDef (Version, 0);
  end;

begin
  Major   := ExtractNumber (Version);
  Minor   := ExtractNumber (Version);
  Release := ExtractNumber (Version);
  Build   := ExtractNumber (Version);
end;


end.
